@@ -1,4 +1,4 @@
-use crate::rust::vec::Vec;
+use alloc::vec::Vec;
 use crate::elements;
 use super::{
 	import,
@@ -211,11 +211,11 @@ impl<F> ModuleBuilder<F> where F: Invoke<elements::Module> {
 	/// Push linear memory region
 	pub fn push_memory(&mut self, mut memory: memory::MemoryDefinition) -> u32 {
 		let entries = self.module.memory.entries_mut();
-		entries.push(elements::MemoryType::new(memory.min, memory.max, false));
+		entries.push(elements::MemoryType::new(memory.min, memory.max));
 		let memory_index = (entries.len() - 1) as u32;
 		for data in memory.data.drain(..) {
 			self.module.data.entries_mut()
-				.push(elements::DataSegment::new(memory_index, Some(data.offset), data.values, false))
+				.push(elements::DataSegment::new(memory_index, Some(data.offset), data.values))
 		}
 		memory_index
 	}
@@ -227,7 +227,7 @@ impl<F> ModuleBuilder<F> where F: Invoke<elements::Module> {
 		let table_index = (entries.len() - 1) as u32;
 		for entry in table.elements.drain(..) {
 			self.module.element.entries_mut()
-				.push(elements::ElementSegment::new(table_index, Some(entry.offset), entry.values, false))
+				.push(elements::ElementSegment::new(table_index, Some(entry.offset), entry.values))
 		}
 		table_index
 	}
