@@ -1,7 +1,7 @@
-use crate::rust::{vec::Vec, borrow::ToOwned, string::String, cmp};
+use alloc::{borrow::ToOwned, vec::Vec, string::String};
 use crate::io;
 
-use super::{Deserialize, Serialize, Error, Uint32, External};
+use super::{deserialize_buffer, serialize, Deserialize, Serialize, Error, Uint32, External};
 use super::section::{
 	Section, CodeSection, TypeSection, ImportSection, ExportSection, FunctionSection,
 	GlobalSection, TableSection, ElementSection, DataSection, MemorySection,
@@ -10,6 +10,8 @@ use super::section::{
 use super::name_section::NameSection;
 use super::reloc_section::RelocSection;
 
+use core::cmp;
+
 const WASM_MAGIC_NUMBER: [u8; 4] = [0x00, 0x61, 0x73, 0x6d];
 
 /// WebAssembly module
@@ -51,6 +53,16 @@ impl Module {
 		}
 	}
 
+	/// Construct a module from a slice.
+	pub fn from_bytes<T: AsRef<[u8]>>(input: T) -> Result<Self, Error> {
+		Ok(deserialize_buffer::<Module>(input.as_ref())?)
+	}
+
+	/// Serialize a module to a vector.
+	pub fn to_bytes(self) -> Result<Vec<u8>, Error> {
+		Ok(serialize::<Module>(self)?)
+	}
+
 	/// Destructure the module, yielding sections
 	pub fn into_sections(self) -> Vec<Section> {
 		self.sections
@@ -73,6 +85,32 @@ impl Module {
 		&mut self.sections
 	}
 
+	/// Insert a section, in the correct section ordering. This will fail with an error,
+	/// if the section can only appear once.
+	pub fn insert_section(&mut self, section: Section) -> Result<(), Error> {
+		let sections = self.sections_mut();
+
+		// Custom sections can be inserted anywhere. Lets always insert them last here.
+		if section.order() == 0 {
+			sections.push(section);
+			return Ok(());
+		}
+
+		// Check if the section already exists.
+		if sections.iter().position(|s| s.order() == section.order()).is_some() {
+			return Err(Error::DuplicatedSections(section.order()));
+		}
+
+		// Assume that the module is already well-ordered.
+		if let Some(pos) = sections.iter().position(|s| section.order() < s.order()) {
+			sections.insert(pos, section);
+		} else {
+			sections.push(section);
+		}
+
+		Ok(())
+	}
+
 	/// Code section reference, if any.
 	pub fn code_section(&self) -> Option<&CodeSection> {
 		for section in self.sections() {
@@ -250,11 +288,8 @@ impl Module {
 				return
 			}
 		}
-		let insert_before = self.sections().iter().enumerate()
-			.filter_map(|(i, s)| if s.order() > 0x8 { Some(i) } else { None })
-			.next()
-			.unwrap_or(0);
-		self.sections_mut().insert(insert_before, Section::Start(new_start));
+		// This should not fail, because we update the existing section above.
+		self.insert_section(Section::Start(new_start)).expect("insert_section should not fail");
 	}
 
 	/// Removes the module's start section.
@@ -324,6 +359,22 @@ impl Module {
 		None
 	}
 
+	/// True if a name section is present.
+	///
+	/// NOTE: this can return true even if the section was not parsed, hence `names_section()` may return `None`
+	///       even if this returns `true`
+	pub fn has_names_section(&self) -> bool {
+		self.sections().iter().any(|e| {
+			match e {
+				// The default case, when the section was not parsed
+				Section::Custom(custom) => custom.name() == "name",
+				// This is the case, when the section was parsed
+				Section::Name(_) => true,
+				_ => false,
+			}
+		})
+	}
+
 	/// Functions signatures section reference, if any.
 	///
 	/// NOTE: name section is not parsed by default so `names_section` could return None even if name section exists.
@@ -593,7 +644,7 @@ pub fn peek_size(source: &[u8]) -> usize {
 
 #[cfg(test)]
 mod integration_tests {
-	use super::super::{deserialize_file, serialize, deserialize_buffer, Section};
+	use super::super::{deserialize_file, serialize, deserialize_buffer, Section, TypeSection, FunctionSection, ExportSection, CodeSection};
 	use super::Module;
 
 	#[test]
@@ -785,10 +836,11 @@ mod integration_tests {
 		assert!(found_section, "Name section should be present in dedicated example");
 	}
 
+	// This test fixture has FLAG_SHARED so it depends on atomics feature.
 	#[test]
-	fn varuint1_case() {
-		let _module = deserialize_file("./res/cases/v1/varuint1_1.wasm")
-			.expect("Maybe shouldn't be deserialized");
+	fn shared_memory_flag() {
+		let module = deserialize_file("./res/cases/v1/varuint1_1.wasm");
+		assert_eq!(module.is_ok(), cfg!(feature="atomics"));
 	}
 
 
@@ -851,4 +903,69 @@ mod integration_tests {
         let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
         assert_eq!(sections, vec![1, 2, 3, 6, 7, 8, 9, 11, 12, 0]);
     }
+
+    #[test]
+    fn names_section_present() {
+        let mut module = deserialize_file("./res/cases/v1/names.wasm").expect("failed to deserialize");
+
+        // Before parsing
+        assert!(module.names_section().is_none());
+        assert!(module.names_section_mut().is_none());
+        assert!(module.has_names_section());
+
+        // After parsing
+        let mut module = module.parse_names().expect("failed to parse names section");
+        assert!(module.names_section().is_some());
+        assert!(module.names_section_mut().is_some());
+        assert!(module.has_names_section());
+    }
+
+    #[test]
+    fn names_section_not_present() {
+        let mut module = deserialize_file("./res/cases/v1/test.wasm").expect("failed to deserialize");
+
+        // Before parsing
+        assert!(module.names_section().is_none());
+        assert!(module.names_section_mut().is_none());
+        assert!(!module.has_names_section());
+
+        // After parsing
+        let mut module = module.parse_names().expect("failed to parse names section");
+        assert!(module.names_section().is_none());
+        assert!(module.names_section_mut().is_none());
+        assert!(!module.has_names_section());
+    }
+
+    #[test]
+    fn insert_sections() {
+        let mut module = Module::default();
+
+        assert!(module.insert_section(Section::Function(FunctionSection::with_entries(vec![]))).is_ok());
+        // Duplicate.
+        assert!(module.insert_section(Section::Function(FunctionSection::with_entries(vec![]))).is_err());
+
+        assert!(module.insert_section(Section::Type(TypeSection::with_types(vec![]))).is_ok());
+        // Duplicate.
+        assert!(module.insert_section(Section::Type(TypeSection::with_types(vec![]))).is_err());
+
+        assert!(module.insert_section(Section::Export(ExportSection::with_entries(vec![]))).is_ok());
+        // Duplicate.
+        assert!(module.insert_section(Section::Export(ExportSection::with_entries(vec![]))).is_err());
+
+        assert!(module.insert_section(Section::Code(CodeSection::with_bodies(vec![]))).is_ok());
+        // Duplicate.
+        assert!(module.insert_section(Section::Code(CodeSection::with_bodies(vec![]))).is_err());
+
+        // Try serialisation roundtrip to check well-orderedness.
+        let serialized = serialize(module).expect("serialization to succeed");
+        assert!(deserialize_buffer::<Module>(&serialized).is_ok());
+    }
+
+    #[test]
+    fn serialization_roundtrip() {
+        let module = deserialize_file("./res/cases/v1/test.wasm").expect("failed to deserialize");
+        let module_copy = module.clone().to_bytes().expect("failed to serialize");
+        let module_copy = Module::from_bytes(&module_copy).expect("failed to deserialize");
+        assert_eq!(module, module_copy);
+    }
 }
